@@ -52,6 +52,7 @@ CREATE TABLE IF NOT EXISTS graphs (
     meta         TEXT,
     created_at   INT UNSIGNED NOT NULL,
     updated_at   INT UNSIGNED NOT NULL,
+    timestamp    INT UNSIGNED DEFAULT NULL,
     PRIMARY KEY (id),
     UNIQUE  (service_name, section_name, graph_name)
 )  ENGINE=InnoDB DEFAULT CHARSET=utf8
@@ -95,6 +96,43 @@ CREATE TABLE IF NOT EXISTS complex_graphs (
     UNIQUE  (service_name, section_name, graph_name)
 )  ENGINE=InnoDB DEFAULT CHARSET=utf8
 EOF
+
+        $dbh->do(<<EOF);
+CREATE TABLE IF NOT EXISTS vrules (
+    id           INT UNSIGNED NOT NULL AUTO_INCREMENT,
+    graph_path   VARCHAR(255) NOT NULL COLLATE utf8_bin,
+    time         INT UNSIGNED NOT NULL,
+    color        VARCHAR(255) NOT NULL DEFAULT '#FF0000',
+    description  TEXT,
+    dashes       VARCHAR(255) NOT NULL DEFAULT '',
+    PRIMARY KEY (id),
+    INDEX time_graph_path (time, graph_path)
+)  ENGINE=InnoDB DEFAULT CHARSET=utf8
+EOF
+
+        {
+            my $sth = $dbh->column_info(undef,undef,"vrules",undef);
+            my $columns = $sth->fetchall_arrayref(+{ COLUMN_NAME => 1 });
+            my %graphs_columns;
+            $graphs_columns{$_->{COLUMN_NAME}} = 1 for @$columns;
+            if ( ! exists $graphs_columns{dashes} ) {
+                infof("add new column 'dashes'");
+                $dbh->do(q{ALTER TABLE vrules ADD dashes VARCHAR(255) NOT NULL DEFAULT ''});
+            }
+        }
+
+        # timestamp
+        {
+            my $sth = $dbh->column_info(undef,undef,"graphs",undef);
+            my $columns = $sth->fetchall_arrayref(+{ COLUMN_NAME => 1 });
+            my %graphs_columns;
+            $graphs_columns{$_->{COLUMN_NAME}} = 1 for @$columns;
+            if ( ! exists $graphs_columns{timestamp} ) {
+                infof("add new column 'timestamp'");
+                $dbh->do(q{ALTER TABLE graphs ADD timestamp INT UNSIGNED DEFAULT NULL});
+            }
+        }
+
         return;
     };
 }
